import jm.music.data.*;
import jm.JMC;
import jm.util.*;
import jm.music.tools.*;

/**
 * Class that represents elements of the song
 * amazing grace version 2
 * @author Mark Guzdial
 * @author Barb Ericson
 */
public class AmazingGraceSongElement2 {
  
  /** the next element */
  private AmazingGraceSongElement2 next;
  /** my part in the song */
  private Part myPart;
  
  /**
   * When we make a new element, the next part is 
   * empty, and ours is a blank new part
   */
  public AmazingGraceSongElement2() {
    this.next = null;
    this.myPart = new Part();
  }
  
  /** 
   * setPhrase takes a phrase and makes it the one for this 
   * element at the desired start time with the given instrument
   * @param myPhrase the phrase to use
   * @param startTime the time to start
   * @param instrument the instrument to use
   */
  public void setPhrase(Phrase myPhrase, double startTime, 
                        int instrument){
    
    /* Phrases get returned from phrase1() and phrase2() with 
     * default (0.0) startTime
     * We can set it here with whatever setPhrase gets as input */
    myPhrase.setStartTime(startTime);
    this.myPart.addPhrase(myPhrase);
    
    /* In MVAmazingGraceSong, we did this when we initialized
     * the part. But we CAN do it later */
    this.myPart.setInstrument(instrument);
  }
  
  /**
   * method that returns the first 
   * phrase of Amazing Grace
   * @return phrase1
   */
  public Phrase phrase1() {
    double[] phrase1Data = { 
      JMC.G4, JMC.QN,
      JMC.C5, JMC.HN, JMC.E5,JMC.EN, JMC.C5,JMC.EN,
      JMC.E5,JMC.HN,JMC.D5,JMC.QN,
      JMC.C5,JMC.HN,JMC.A4,JMC.QN,
      JMC.G4,JMC.HN,JMC.G4,JMC.EN,JMC.A4,JMC.EN,
      JMC.C5,JMC.HN,JMC.E5,JMC.EN,JMC.C5,JMC.EN,
      JMC.E5,JMC.HN,JMC.D5,JMC.EN,JMC.E5,JMC.EN,
      JMC.G5,JMC.DHN
    };
    
    Phrase myPhrase = new Phrase();
    myPhrase.addNoteList(phrase1Data);
    return myPhrase;
  }
  
  /**
   * method that returns phrase 2
   * @return phrase 2
   */
  public Phrase phrase2() {
    double[] phrase2Data = {
      JMC.G5,JMC.HN,JMC.E5,JMC.EN,JMC.G5,JMC.EN,
      JMC.G5,JMC.HN,JMC.E5,JMC.EN,JMC.C5,JMC.EN,
      JMC.E5,JMC.HN,JMC.D5,JMC.QN,
      JMC.C5,JMC.HN,JMC.A4,JMC.QN,
      JMC.G4,JMC.HN,JMC.G4,JMC.EN,JMC.A4,JMC.EN,
      JMC.C5,JMC.HN,JMC.E5,JMC.EN,JMC.C5,JMC.EN,
      JMC.E5,JMC.HN,JMC.D5,JMC.QN,
      JMC.C5,JMC.DHN
    };
    
    Phrase myPhrase = new Phrase();
    myPhrase.addNoteList(phrase2Data);
    return myPhrase;
  }
  
  /**
   * Method to set the next element
   * @param nextOne the next element
   */
  public void setNext(AmazingGraceSongElement2 nextOne) {
    this.next = nextOne;
  }
  
  /**
   * Method to get the next element
   * @return the next element
   */
  public AmazingGraceSongElement2 getNext() {
    return this.next;
  }
  
  /** 
   * We could just access myPart directly
   * but we can CONTROL access by using a method
   * (called an accessor)
   * @return the part
   */
  public Part part() {
    return this.myPart;
  }
  
  /** Why do we need this?
    * If we want one piece to start after another, we need
    * to know when the last one ends.
    * Notice: It's the phrase that knows the end time.
    *    We have to ask the part for its phrase (assuming only one)
    *    to get the end time.
    * @return the end time
    */
  public double getEndTime() {
    return this.myPart.getPhrase(0).getEndTime();
  }
  
  /** We need setChannel because each part has to be in its
    * own channel if it has different start times.
    * So, we'll set the channel when we assemble the score.
    * @param the channel to use
    */
  public void setChannel(int channel) {
    myPart.setChannel(channel);
  }
  
  /**
   * Method to show from this element on
   */
  public void showFromMeOn() {
    // Make the score that we'll assemble the elements into
    // We'll set it up with the time signature and tempo we like
    Score myScore = new Score("Amazing Grace");
    myScore.setTimeSignature(3,4);
    myScore.setTempo(120.0);
    
    // Each element will be in its own channel
    int channelCount = 1;
    
    // Start from this element (this)
    AmazingGraceSongElement2 current = this;
    
    // While we're not through...
    while (current != null) {
      
      // Set the channel, increment the channel, then add it in.
      current.setChannel(channelCount);
      channelCount = channelCount + 1;
      myScore.addPart(current.part());
      
      // Now, move on to the next element
      current = current.getNext();
    }
    
    // At the end, let's see it!
    View.notate(myScore);
  }
  
}
