/** 
 * NBD Agent
 * @author Cassy Romano
 * @author Barb Ericson
 */
public class NBDAgent extends Agent {
  
  /** pictures to use */
  private static Picture hunter1, blank;
  
  ////////////////////////////// Constructors ////////////////////////
  // Copy this section AS-IS into subclasses, but rename 
  
  /**
   * Constructor that takes the model display (the original
   * position will be randomly assigned) and a simulation
   * @param modelDisplayer thing that displays the model
   * @param thisSim my simulation
   */
  public NBDAgent (ModelDisplay modelDisplayer,
                   Simulation thisSim) {
    super(randNumGen.nextInt(modelDisplayer.getWidth()),
          randNumGen.nextInt(modelDisplayer.getHeight()),
          modelDisplayer, thisSim);
  }
  
  /** Constructor that takes the x and y, a model
    * display to draw it on, and a simulation
    * @param x the starting x position
    * @param y the starting y position
    * @param modelDisplayer the thing that displays the model
    * @param thisSim my simulation
    */
  public NBDAgent (int x, int y, ModelDisplay modelDisplayer, 
                   Simulation thisSim) {
    // let the parent constructor handle it
    super(x,y,modelDisplayer,thisSim);
  }
  
  /**
   * To initialize the simulation
   * @param thisSim the simulation
   */
  public void init(Simulation thisSim) {
    if (hunter1 == null) { //Initialize
      hunter1 = new Picture(FileChooser.getMediaPath("hunter.jpg"));
      blank = new Picture(FileChooser.getMediaPath("blank.jpg"));
    }
    // set the starting picture
    this.setPicture(hunter1);
    
    // Normal initialization
    super.init(thisSim);
    
    // Move the agent
    this.moveTo(20,20);
  }
  
  /**
   * Method that provides the act behavior
   * @param t the current timestep
   */
  public void act(int t) {    
    if (t < 10) {
      this.setPicture(blank);
    }
    
    else if (t>= 10) {
      this.setPicture(hunter1);
      this.forward(randNumGen.nextInt(5));
    }
    
  } // end act()
  
}
